=begin

= apache/asp-run.rb

Copyright (C) 2003 Gregoire Lejeune <gregoire.lejeune@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

== Overview

Apache::AspRun handles eRuby files by Asp.

== Example of .htaccess

AuthName "Ruby/ASP"

RubySetEnv Global /var/www/tests/ruby-asp/global-asa

RubyRequire apache/aspruby-run
<Files ~ (\.asp)>
  SetHandler ruby-object
  RubyHandler Apache::AspRun.instance
</Files>

=end

require "singleton"
require "tempfile"
require 'net/smtp'
require "asp/common"
require "asp/compile"
require "asp/globalasa"
require "asp/response"
require "asp/request"
require "asp/session"
require "asp/application"
require "asp/server"
require "asp/goto"

module Apache
  class AspRun
    include Singleton

    def handler(r)
      if r.method_number == M_OPTIONS
        r.allowed |= (1 << M_GET)
        r.allowed |= (1 << M_POST)
        return DECLINED
      end
      if r.finfo.mode == 0
        return NOT_FOUND
      end

      s = r.server( )
#      c = r.connection( )
#      t = r.headers_in( )
      
      r.setup_cgi_env
      
      btime = Time::now
      
      ## Add global-asa path to library search path
#      $:.untaint
#      $:.unshift( File.join( ENV["Global"] ) )

      ## Load global.asa
      oGAsa = Asp::GlobalASA::new( ENV["Global"], r ) 
      if oGAsa.getAsaCode == false 
        print __FILE__," L",__LINE__," :: Call ASP.Error\n";
        return OK
      end

      $Request = Asp::Request::new( r )
      $Response = Asp::Response::new( r )
      $Session = Asp::Session::new( r )
      $Server = Asp::Server::new( r )
      $Application = Asp::Application::new( )

      filename = r.filename.dup
      filename.untaint
      
      run_state = 0
      code = compile(filename)
	  ## DEBUG #  print "<!-- #{code} -->\n"
      begin
        run_state = 1
        prerun(r)
        run_state = 2
        run(code, filename)
		if $Response.bEnd == 0 and $Server.bEnd == 0
          run_state = 3
          postrun(r)
		end

        etime = Time::now
        ctime = etime - btime
        print "<!-- Ruby/ASP v",Asp::Common::version," served page in ",ctime," seconds -->"
      rescue NameError, SyntaxError, ZeroDivisionError, RuntimeError, ArgumentError, SecurityError, LoadError => e
        debug(e, r, code, filename, run_state)
      end

      return OK
    end

    private

    def initialize
      @compiler = AspCompiler.new
      @compiler.put_cmd = 'print'
    end

    def compile(filename)
      open(filename) do |f|
        return @compiler.compile(f.read)
      end
    end

    def prerun(r)
      if self.private_methods.member?( "script_OnStart" ) == true
        script_OnStart
      end
      Apache.chdir_file(r.filename)
    end

    def run(code, filename)
      tmpdir = Asp::Common::stateDir
      file = Tempfile.new(File.basename(filename) + ".", tmpdir)
      begin
        file.print(code)
        file.close

        frame_start
		label(:a){
          load(file.path, true)
		}
		label(:e) {}
		frame_end
      ensure
        file.close(true)
      end
    end

    def postrun(r)
      if self.private_methods.member?( "script_OnEnd" ) == true
        script_OnEnd
      end

      r.content_type = format( "%s; charset=%s", $Response.contentType, $Response.charset )
      $Response.getHeader.each_pair{ |key,value|
        r.headers_out[key] = value
      }
      $Response.addCookiesHeader.each { |cookie|
        r.headers_out.add( "Set-Cookie", cookie )
      }
      sessionCookie = $Session.addSessionCookieHeader
      if sessionCookie.nil? == false
        r.headers_out.add( "Set-Cookie", sessionCookie )
      end
      r.send_http_header
    end

    def debug(e, r, code, filename, run_state)

      error_page  = ENV["ErrorFile"].untaint
      smtp_server = ENV["MailHost"].untaint
      smtp_port   = ENV["MailPort"].untaint || 25
      smtp_from   = ENV["MailErrorsFrom"].untaint
      smtp_to     = ENV["MailErrorsTo"].untaint
      mail_send   = false

      if smtp_server.nil? == false and smtp_from.nil? == false and smtp_to.nil? == false
        smtp = Net::SMTP.new( smtp_server, smtp_port )
        smtp.start()
        smtp.ready( smtp_from, smtp_to ) do |m|
          m.write "From: \"WWW Error Report\" <#{smtp_from}>\r\n"
          m.write "To: \"WWW Error Reporting\" <#{smtp_to}>\r\n"
          m.write "Subject: WWW Error Report: #{r.unparsed_uri}\r\n"
          m.write "\r\n"
          m.puts "Ruby/Asp v#{Asp::Common::version}"
          m.puts "ERROR State ##{run_state} : " + e.message
          m.puts "----------------------------------------------------------"
          m.puts "File " + filename
          m.puts "----------------------------------------------------------"
          i = 0
          r.escape_html(code).each_line { |l|
            i += 1
            m.puts( format( "%04d : %s", i, l.chomp ) )
          }
          m.puts "----------------------------------------------------------"
        end # smtp.ready()

        mail_send = true
      end

      if error_page.nil? == true
        print "<DIV STYLE='background-color: #ffffff; font-family: sans-serif; border: 1px solid #646464; margin-left: 1em; margin-right: 2em; padding: 0.5em;'>\n"
        print "<B>Ruby/Asp v",Asp::Common::version,"</B><BR>\n"
        print "ERROR State #",run_state," : \n"
        print "<B>",r.escape_html(e.message),"</B><BR>\n"
        print "<PRE STYLE='background-color: #e6e6e6; border: 1px solid #646464; margin-left: 1em; margin-right: 2em; padding: 0.5em;'>\n"
        print "<B>File ",r.uri,"</B> (ASP to Ruby Script)\n"
        i = 0
        r.escape_html(code).each_line { |l|
          i += 1
          printf( "%04d : %s\n", i, l.chomp )
        }
        print "</PRE>\n"
        print "Bad things happened. An email has been sent to #{smtp_to} with diagnostic information and details of your request.\n" if mail_send == true
        print "</DIV>\n"
      else
        r.status = 302
        r.headers_out["location"] = error_page
        r.send_http_header
      end

    end

  end
end
